@@ -20,7 +20,7 @@
 #include "filehash.h"
 #include "verify.h"
 
-static char fio_version_string[] = "fio 1.28";
+static char fio_version_string[] = "fio 1.30";
 
 #define FIO_RANDSEED		(0xb1899bedUL)
 
@@ -233,20 +233,6 @@ static int fixup_options(struct thread_data *td)
 	}
 #endif
 
-#ifndef FIO_HAVE_CPU_AFFINITY
-	if (td->o.gtod_cpu) {
-		log_err("fio: platform must support CPU affinity for"
-			"gettimeofday() offloading\n");
-		return 1;
-	}
-#endif
-
-	if (read_only && td_write(td)) {
-		log_err("fio: job <%s> has write bit set, but fio is in"
-			" read-only mode\n", td->o.name);
-		return 1;
-	}
-
 	if (o->write_iolog_file && o->read_iolog_file) {
 		log_err("fio: read iolog overrides write_iolog\n");
 		free(o->write_iolog_file);
@@ -364,8 +350,12 @@ static int fixup_options(struct thread_data *td)
 	if (td->o.verify != VERIFY_NONE)
 		td->o.refill_buffers = 1;
 
-	if (td->o.pre_read)
+	if (td->o.pre_read) {
 		td->o.invalidate_cache = 0;
+		if (td->io_ops->flags & FIO_PIPEIO)
+			log_info("fio: cannot pre-read files with an IO engine"
+				 " that isn't seekable. Pre-read disabled.\n");
+	}
 
 	return 0;
 }
@@ -920,6 +910,7 @@ static void usage(const char *name)
 	printf("\t--section=name\tOnly run specified section in job file\n");
 	printf("\t--alloc-size=kb\tSet smalloc pool to this size in kb"
 		" (def 1024)\n");
+	printf("\nFio was written by Jens Axboe <jens.axboe@oracle.com>\n");
 }
 
 #ifdef FIO_INC_DEBUG
@@ -1156,7 +1147,7 @@ int parse_options(int argc, char *argv[])
 		if (dump_cmdline)
 			return 0;
 
-		log_err("No jobs defined(s)\n");
+		log_err("No jobs defined(s)\n\n");
 		usage(argv[0]);
 		return 1;
 	}
