@@ -575,6 +575,34 @@ static int str_gtod_cpu_cb(void *data, int *il)
 	return 0;
 }
 
+static int rw_verify(struct fio_option *o, void *data)
+{
+	struct thread_data *td = data;
+
+	if (read_only && td_write(td)) {
+		log_err("fio: job <%s> has write bit set, but fio is in"
+			" read-only mode\n", td->o.name);
+		return 1;
+	}
+
+	return 0;
+}
+
+static int gtod_cpu_verify(struct fio_option *o, void *data)
+{
+#ifndef FIO_HAVE_CPU_AFFINITY
+	struct thread_data *td = data;
+
+	if (td->o.gtod_cpu) {
+		log_err("fio: platform must support CPU affinity for"
+			"gettimeofday() offloading\n");
+		return 1;
+	}
+#endif
+
+	return 0;
+}
+
 #define __stringify_1(x)	#x
 #define __stringify(x)		__stringify_1(x)
 
@@ -648,6 +676,7 @@ static struct fio_option options[] = {
 		.off1	= td_var_offset(td_ddir),
 		.help	= "IO direction",
 		.def	= "read",
+		.verify	= rw_verify,
 		.posval = {
 			  { .ival = "read",
 			    .oval = TD_DDIR_READ,
@@ -931,6 +960,13 @@ static struct fio_option options[] = {
 		.help	= "Issue fsync for writes every given number of blocks",
 		.def	= "0",
 	},
+	{
+		.name	= "fdatasync",
+		.type	= FIO_OPT_INT,
+		.off1	= td_var_offset(fdatasync_blocks),
+		.help	= "Issue fdatasync for writes every given number of blocks",
+		.def	= "0",
+	},
 	{
 		.name	= "direct",
 		.type	= FIO_OPT_BOOL,
@@ -1029,6 +1065,16 @@ static struct fio_option options[] = {
 #endif
 		  },
 	},
+	{
+		.name	= "iomem_align",
+		.alias	= "mem_align",
+		.type	= FIO_OPT_INT,
+		.off1	= td_var_offset(mem_align),
+		.minval	= 0,
+		.help	= "IO memory buffer offset alignment",
+		.def	= "0",
+		.parent	= "iomem",
+	},
 	{
 		.name	= "verify",
 		.type	= FIO_OPT_STR,
@@ -1503,6 +1549,14 @@ static struct fio_option options[] = {
 		.type	= FIO_OPT_INT,
 		.cb	= str_gtod_cpu_cb,
 		.help	= "Setup dedicated gettimeofday() thread on this CPU",
+		.verify	= gtod_cpu_verify,
+	},
+	{
+		.name	= "continue_on_error",
+		.type	= FIO_OPT_BOOL,
+		.off1	= td_var_offset(continue_on_error),
+		.help	= "Continue on non-fatal errors during I/O",
+		.def	= "0",
 	},
 	{
 		.name = NULL,
