@@ -46,7 +46,10 @@ static void show_option_range(struct fio_option *o)
 	if (!o->minval && !o->maxval)
 		return;
 
-	printf("%20s: min=%d, max=%d\n", "range", o->minval, o->maxval);
+	printf("%20s: min=%d", "range", o->minval);
+	if (o->maxval)
+		printf(", max=%d", o->maxval);
+	printf("\n");
 }
 
 static void show_option_values(struct fio_option *o)
@@ -425,6 +428,12 @@ static int __handle_option(struct fio_option *o, const char *ptr, void *data,
 		ret = 1;
 	}
 
+	if (ret)
+		return ret;
+
+	if (o->verify)
+		ret = o->verify(o, data);
+
 	return ret;
 }
 
@@ -664,7 +673,6 @@ static void show_option_help(struct fio_option *o)
 	const char *typehelp[] = {
 		"string (opt=bla)",
 		"string with possible k/m/g postfix (opt=4k)",
-		"string with range and postfix (opt=1k-4k)",
 		"string with time postfix (opt=10s)",
 		"string (opt=bla)",
 		"string with dual range (opt=1k-4k,4k-8k)",
